using System;
using System.Collections.Generic;
using System.Text;

namespace M_C_V
{
    public abstract class AbstractMCVView : MCV.IViewable, MCV.IViewableEvents
    {
        private bool isEnabled = true;
        private string disableMessage = "";

        public bool Enabled
        {
            get { return this.isEnabled; }
            set { this.isEnabled = value; }
        }
        public string DisableMessage
        {
            get { return this.disableMessage; }
            set
            {
                if (value != null)
                {
                    this.disableMessage = value;
                }
                else
                {
                    this.disableMessage = "";
                }
            }
        }
        // abstract IViewable methods
        public abstract bool Clear();
        public abstract bool Refresh();
        public abstract bool Reset();

        // concrete IViewable methods
        public virtual bool Enable()
        {
            this.isEnabled = true;
            return true;
        }
        // reset and clear
        public virtual bool Disable(string message)
        {
            this.isEnabled = false;
            if (message != null)
            {
                disableMessage = message;
            }
            else
            {
                disableMessage = "";
            }
            this.Reset();
            this.Clear();
            return true;
        }


        // Provide methods that return delegates for IViewable handlers
        public virtual MCV.DisableHandler GetDisableHandler()
        {
            return new MCV.DisableHandler(AbstractMCVView_OnDisableEvent);
        }
        public virtual MCV.EnableHandler GetEnableHandler()
        {
            return new MCV.EnableHandler(AbstractMCVView_OnEnableEvent);
        }
        public virtual MCV.ClearHandler GetClearHandler()
        {
            return new MCV.ClearHandler(AbstractMCVView_OnClearEvent);
        }
        public virtual MCV.RefreshHandler GetRefreshHandler()
        {
            return new MCV.RefreshHandler(AbstractMCVView_OnRefreshEvent);

        }
        public virtual MCV.ResetHandler GetResetHandler()
        {
            return new MCV.ResetHandler(AbstractMCVView_OnResetEvent);
        }

        // Actual event handlers
        public virtual void AbstractMCVView_OnDisableEvent(object source, MCV.DisableEventArgs e)
        {
            this.Disable(e.Message);
        }
        public virtual void AbstractMCVView_OnEnableEvent(object source, EventArgs e)
        {
            this.Enable();
        }
        public virtual void AbstractMCVView_OnClearEvent(object source, EventArgs e)
        {
            this.Clear();
        }
        public virtual void AbstractMCVView_OnRefreshEvent(object source, EventArgs e)
        {
            this.Refresh();
        }
        public virtual void AbstractMCVView_OnResetEvent(object source, EventArgs e)
        {
            this.Reset();
        }
    }
}
