using System;
using System.Collections.Generic;
using System.Text;

namespace M_C_V
{
        // Model that Encapsulate algorithms here
        // Cannot use static methods to implement interface so this is a class
        // Abstract interaction with controller by implementing Morgtage.ICalculate.Calculate
        public class Model : object, Mortgage.ICalculate
        {
            /// <summary>
            /// Class Model.cs
            /// jlouie 07.07.02
            /// updated 10.31.06 to return immutable Mortgage struct
            /// Mortgage struct is nested in this class
            /// Adapted from Model.java
            /// "Visual Cafe for Java Explorer, Database Development Edition"
            /// William Brogden, Jeffrey A. Louie, and Ed Tittel, Coriolis, 1998, 585pp.
            /// Supplied "as is"
            /// No warranty is expressed or implied
            /// This code is for instructional use only
            /// </summary>
            /// 

            // Model.GetCalculation returns immutable Mortgage structure
            // takes immutable Parameters structure
            // the actual amortization algorithm
            // m= P*i(1-(1+i)^-N)
            // i=r/1200
            // result= 0 --> marks error 
            public Mortgage.Calculation Calculate(Mortgage.Parameters m)
            {
                return Calculate(m.Principal, m.Interest, m.Months, m.Payment, m.Target);
            }

            private Mortgage.Calculation Calculate(double principal, double interest, int months, double payment, int target)
            {
                double[] arrayDb = new double[4];
                string message = "Invalid Result";

                // store input into array of double
                arrayDb[Mortgage.PRINCIPAL] = principal;
                arrayDb[Mortgage.INTEREST] = interest;
                arrayDb[Mortgage.MONTHS] = (double)months;
                arrayDb[Mortgage.PAYMENT] = payment;

                // clear target if target is valid
                if ((target >= Mortgage.PRINCIPAL) && (target <= Mortgage.PAYMENT))
                {
                    arrayDb[target] = 0;
                }
                else // return on error
                {
                    target = Mortgage.INVALID;
                    message = "Invalid target.";
                    return new Mortgage.Calculation(arrayDb[Mortgage.PRINCIPAL], arrayDb[Mortgage.INTEREST], (int)arrayDb[Mortgage.MONTHS], arrayDb[Mortgage.PAYMENT], Mortgage.INVALID, message);
                }

                // validate input

                // one, and only one, "value" must be zero --> target
                int zeros = 0;
                for (int index = 0; index < 4; index++)
                {
                    if (arrayDb[index] == 0)
                    {
                        zeros++;
                        target = index;
                    }
                }
                if (zeros > 1)
                {
                    message = "A necessary parameter is zero.";
                    return new Mortgage.Calculation(arrayDb[Mortgage.PRINCIPAL], arrayDb[Mortgage.INTEREST], (int)arrayDb[Mortgage.MONTHS], arrayDb[Mortgage.PAYMENT], Mortgage.INVALID, message);
                }

                // validate interest
                if (interest > 100 || interest < 0)
                {
                    message = "Invalid interest.";
                    return new Mortgage.Calculation(arrayDb[Mortgage.PRINCIPAL], arrayDb[Mortgage.INTEREST], (int)arrayDb[Mortgage.MONTHS], arrayDb[Mortgage.PAYMENT], Mortgage.INVALID, message);
                }
                // validate months
                if (months < 0)
                {
                    message = "Invalid months.";
                    return new Mortgage.Calculation(arrayDb[Mortgage.PRINCIPAL], arrayDb[Mortgage.INTEREST], (int)arrayDb[Mortgage.MONTHS], arrayDb[Mortgage.PAYMENT], Mortgage.INVALID, message);
                }
                // validate principal
                if (principal < 0)
                {
                    message = "Invalid principal.";
                    return new Mortgage.Calculation(arrayDb[Mortgage.PRINCIPAL], arrayDb[Mortgage.INTEREST], (int)arrayDb[Mortgage.MONTHS], arrayDb[Mortgage.PAYMENT], Mortgage.INVALID, message);
                }
                // validate payment
                if (payment < 0)
                {
                    message = "Invalid payment.";
                    return new Mortgage.Calculation(arrayDb[Mortgage.PRINCIPAL], arrayDb[Mortgage.INTEREST], (int)arrayDb[Mortgage.MONTHS], arrayDb[Mortgage.PAYMENT], Mortgage.INVALID, message);
                }

                // input parameters appear valid
                // pre-conditions have been met
                // target is valid

                double result = 0;

                double monthlyInterest = arrayDb[Mortgage.INTEREST] / 1200;
                try
                {
                    switch (target)
                    {
                        case Mortgage.PRINCIPAL:  // principal
                            result = 1 + monthlyInterest;
                            result = 1 / Math.Pow(result, arrayDb[Mortgage.MONTHS]);
                            result = ((1 - result) / monthlyInterest * arrayDb[Mortgage.PAYMENT]);
                            break;
                        case Mortgage.INTEREST:  // annual interest
                            // algorithm fails if arrayDb[Model.MONTHS]*arrayDb[Model.PAYMENT] >= arrayDb[Model.PRINCIPAL] !!
                            if ((arrayDb[Mortgage.MONTHS] * arrayDb[Mortgage.PAYMENT]) < arrayDb[Mortgage.PRINCIPAL])
                            {
                                throw new ArithmeticException();
                            }
                            // factor out Interest function, too long
                            result = CalcInterest(arrayDb[Mortgage.PRINCIPAL], arrayDb[Mortgage.MONTHS], arrayDb[Mortgage.PAYMENT]);
                            break;
                        case Mortgage.MONTHS:  // loan period
                            result = (1 - (arrayDb[Mortgage.PRINCIPAL] * (monthlyInterest) / arrayDb[Mortgage.PAYMENT]));
                            result = Math.Log(result);
                            result = -result / Math.Log((1 + monthlyInterest));
                            break;
                        case Mortgage.PAYMENT:  // monthly payments
                            result = 1 + monthlyInterest;
                            result = 1 / Math.Pow(result, arrayDb[Mortgage.MONTHS]);
                            result = (arrayDb[Mortgage.PRINCIPAL] * monthlyInterest) / (1 - result);
                            break;
                        //default:
                    }
                }
                catch
                {
                    result = 0;
                }

                // validate result
                // post-conditions
                if (Double.IsNaN(result))
                {
                    result = 0;
                }
                if (result == 0)
                {
                    message = "Input Error.";
                }
                else // valid result
                {
                    arrayDb[target] = result;
                    message = "";
                }

                return new Mortgage.Calculation(arrayDb[Mortgage.PRINCIPAL], arrayDb[Mortgage.INTEREST], (int)arrayDb[Mortgage.MONTHS], arrayDb[Mortgage.PAYMENT], target, message);
            } // end Calculation method

            // a complex iterative calculation for interest
            // thanks to Dr. W. Carlini (and Newton)for the solution
            // returns zero on error
            // ASSERT (N*m)>=P
            private double CalcInterest(double P, double N, double m)
            {
                double temp = (m / P), answer = (m / P), diff = 100, numerator = 0, denominator = 0,
            accuracy = .00001;
                int index, maxIterations = 1000;
                try
                {
                    for (index = 0; ((diff > accuracy) && (index < maxIterations)); index++)
                    {
                        temp = answer;
                        numerator = (P * temp / m) + Math.Pow((1 + temp), -N) - 1;
                        denominator = (P / m) - N * Math.Pow((1 + temp), (-N - 1));
                        // if (denominator ==0 ){throw new ArithmeticException();}
                        answer = temp - (numerator / denominator);
                        diff = answer - temp;
                        if (diff < 0)
                        {
                            diff = -diff;
                        }
                    }
                    answer *= 1200;
                    // validate answer
                    if ((answer < 0) || Double.IsNaN(answer) ||
            (index == maxIterations))
                    {
                        throw new ArithmeticException();
                    }
                }
                catch
                {
                    answer = 0;
                }
                return answer;
            } // end CalcInterest
        }// end Model
}
