using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

namespace M_C_V
{
    public class MortgageTotalDerivedView : AbstractMCVView
    {
        private Mortgage.Calculation defaultCalculation = new Mortgage.Calculation(0, 0, 0, 0, Mortgage.INVALID, "");
        private Mortgage.Calculation lastCalculation;
        private TextBox tb;

        public Mortgage.Calculation Calculation
        {
            get { return this.lastCalculation; }
        }

        // Constructor
        // ASSERT tb != null
        public MortgageTotalDerivedView(TextBox tb)
        {
            if (tb == null)
            {
                throw new ArgumentNullException();
            }
            this.tb = tb;
            this.lastCalculation = defaultCalculation;
        }

        // implement IViewable methods
        public override bool Clear()
        {
            tb.Clear();
            return true;
        }
        // refresh from LOCAL data
        public override bool Refresh()
        {
            if (lastCalculation.IsValid())
            {
                RefreshView(lastCalculation);
                return true;
            }
            else
            {
                this.Clear();
                return false;
            }
        }
        public override bool Reset()
        {
            this.lastCalculation = defaultCalculation;
            return true;
        }
        // helper method for Refresh
        private void RefreshView(Mortgage.Calculation mc)
        {
            if (Enabled)
            {
                if ((tb != null) && mc.IsValid())
                {
                    lastCalculation = mc;  // store last Mortgage.Calculation
                    double total = mc.Payment * mc.Months;
                    tb.Text = String.Format("${0:N}", total);
                }
                else
                {
                    tb.Text = "Error.";
                }
            }
            else
            {
                tb.Text = DisableMessage;
            }
        }

        public Mortgage.NotifyHandler GetNotifyHandler()
        {
            return new Mortgage.NotifyHandler(MortgageTotalView_OnNotifyEvent);
        }
        public void MortgageTotalView_OnNotifyEvent(object source, Mortgage.MortgageEventArgs e)
        {
            Mortgage.Calculation c = e.calculation;
            // restore from LOCAL data
            RefreshView(c);
        }

    }
}
